//--------------------------------------------------------------
// File     : stm32_ub_scope_Screen.c
// Datum    : 09.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32F4_UB_DAC, TIM, MISC
// Funktion : Screen per Oszi-Bildschirm (im X/Y-Mode)
//            (nur fr Analoge-Oszis bzw Bildroehren)
//
// Hinweise : Pixel X=0, Y=0 = linke untere Ecke
//            kein Blanking-Signal notwendig
//
// Pixel    : Die Auflsung so einstellen, das bei einer Linie
//            keine Einzelpixel mehr zu erkennen sind
//
// Timer    : Den Timer-Interrupt so anpassen, das ein
//            komplettes Bild mit ca 70Hz (15ms) gezeichnet wird
//            (Anzahl der Punkte steht in ScopeScreen.pixel_anz)
//
// CH1 (Y)  : PA5 (0 bis 3,3V)
// CH2 (X)  : PA4 (0 bis 3,3V)
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_scope_screen.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
uint8_t p_ScopeScreen_check_pixel(uint16_t xpos, uint16_t ypos);
int16_t p_ScopeScreen_sgn(int16_t x);
void p_ScopeScreen_init_TIM(void);
void p_ScopeScreen_init_NVIC(void);
void p_ScopeScreen_DrawNextPixel(void);


//--------------------------------------------------------------
// init vom Scope-Screen
//--------------------------------------------------------------
void UB_ScopeScreen_Init(void)
{
  // init aller Variabeln
  ScopeScreen.pixel_anz=0;
  ScopeScreen.akt_pixel_anz=0;
  ScopeScreen.akt_list_pos=0;

  SCOPE_PIXEL_LIST[0].xpos=0;
  SCOPE_PIXEL_LIST[0].ypos=0;

  // init vom DAC
  UB_DAC_Init(DUAL_DAC);

  // DAC auf 0,0
  UB_DAC_SetDAC1u2(0,0);

  // Screen loeschen
  UB_ScopeScreen_Clear();

  // init vom Timer
  p_ScopeScreen_init_TIM();

  // init vom NVIC
  p_ScopeScreen_init_NVIC();

  // Timer enable
  TIM_Cmd(SCOPE_TIMER, ENABLE);
}


//--------------------------------------------------------------
// loescht den kompletten Scope-Screen
//--------------------------------------------------------------
void UB_ScopeScreen_Clear(void)
{
  uint32_t n;

  for(n=0;n<SCOPE_SCREEN_RAM_LEN;n++) {
    SCOPE_SCREEN_RAM[n]=0x00;
  }
}


//--------------------------------------------------------------
// setzt ein Pixel auf dem Scope-Screen
// xpos : [0...SCOPE_SCREEN_MAXX-1]
// ypos : [0...SCOPE_SCREEN_MAXY-1]
//--------------------------------------------------------------
void UB_ScopeScreen_SetPixel(uint16_t xpos, uint16_t ypos)
{
  uint32_t adr,pos;
  uint8_t maske=0;

  if(xpos>=SCOPE_SCREEN_MAXX) xpos=SCOPE_SCREEN_MAXX-1;
  if(ypos>=SCOPE_SCREEN_MAXY) ypos=SCOPE_SCREEN_MAXY-1;

  adr=(ypos*SCOPE_SCREEN_RAM_XLEN)+(xpos>>3);

  pos=(xpos&0x07);
  if(pos==0) maske=0x01;
  if(pos==1) maske=0x02;
  if(pos==2) maske=0x04;
  if(pos==3) maske=0x08;
  if(pos==4) maske=0x10;
  if(pos==5) maske=0x20;
  if(pos==6) maske=0x40;
  if(pos==7) maske=0x80;
  SCOPE_SCREEN_RAM[adr]|=maske;
}


//--------------------------------------------------------------
// loescht ein Pixel auf dem Scope-Screen
// xpos : [0...SCOPE_SCREEN_MAXX-1]
// ypos : [0...SCOPE_SCREEN_MAXY-1]
//--------------------------------------------------------------
void UB_ScopeScreen_ClearPixel(uint16_t xpos, uint16_t ypos)
{
  uint32_t adr,pos;
  uint8_t maske=0;

  if(xpos>=SCOPE_SCREEN_MAXX) xpos=SCOPE_SCREEN_MAXX-1;
  if(ypos>=SCOPE_SCREEN_MAXY) ypos=SCOPE_SCREEN_MAXY-1;

  adr=(ypos*SCOPE_SCREEN_RAM_XLEN)+(xpos>>3);

  pos=(xpos&0x07);
  if(pos==0) maske=0x01;
  if(pos==1) maske=0x02;
  if(pos==2) maske=0x04;
  if(pos==3) maske=0x08;
  if(pos==4) maske=0x10;
  if(pos==5) maske=0x20;
  if(pos==6) maske=0x40;
  if(pos==7) maske=0x80;
  SCOPE_SCREEN_RAM[adr]&=~maske;
}


//--------------------------------------------------------------
// Zeichnet eine Linie
// von x1,y1 nach x2,y2
// (benutzt wird der Bresenham-Algorithmus)
//--------------------------------------------------------------
void UB_ScopeScreen_DrawLine(int16_t x1, int16_t y1, int16_t x2, int16_t y2)
{
  int16_t x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;

  // Entfernung in beiden Dimensionen berechnen
  dx = x2 - x1;
  dy = y2 - y1;

  // Vorzeichen des Inkrements bestimmen
  incx = p_ScopeScreen_sgn(dx);
  incy = p_ScopeScreen_sgn(dy);
  if(dx<0) dx = -dx;
  if(dy<0) dy = -dy;

  // feststellen, welche Entfernung grer ist
  if (dx>dy) {
    // x ist schnelle Richtung
    pdx=incx; pdy=0;    // pd. ist Parallelschritt
    ddx=incx; ddy=incy; // dd. ist Diagonalschritt
    es =dy;   el =dx;   // Fehlerschritte schnell, langsam
  }
  else {
    // y ist schnelle Richtung
    pdx=0;    pdy=incy; // pd. ist Parallelschritt
    ddx=incx; ddy=incy; // dd. ist Diagonalschritt
    es =dx;   el =dy;   // Fehlerschritte schnell, langsam
  }

  // Initialisierungen vor Schleifenbeginn
  x = x1;
  y = y1;
  err = (el>>1);
  UB_ScopeScreen_SetPixel(x,y);

  // Pixel berechnen
  for(t=0; t<el; ++t) // t zaehlt die Pixel, el ist auch Anzahl
  {
    // Aktualisierung Fehlerterm
    err -= es;
    if(err<0) {
      // Fehlerterm wieder positiv (>=0) machen
      err += el;
      // Schritt in langsame Richtung, Diagonalschritt
      x += ddx;
      y += ddy;
    }
    else {
      // Schritt in schnelle Richtung, Parallelschritt
      x += pdx;
      y += pdy;
    }
    UB_ScopeScreen_SetPixel(x,y);
  }
}


//--------------------------------------------------------------
// Zeichnet einen Kreis
// Mittelpunkt = x1,y1
// Radius = radius
// (benutzt wird der Bresenham-Algorithmus)
//--------------------------------------------------------------
void UB_ScopeScreen_DrawCircle(int16_t x0, int16_t y0, int16_t radius)
{
  int16_t f=1-radius, ddF_x=0, ddF_y=-2*radius, x=0, y=radius;

  UB_ScopeScreen_SetPixel(x0, y0 + radius);
  UB_ScopeScreen_SetPixel(x0, y0 - radius);
  UB_ScopeScreen_SetPixel(x0 + radius, y0);
  UB_ScopeScreen_SetPixel(x0 - radius, y0);

  while(x < y)
  {
    if(f >= 0)
    {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x + 1;

    UB_ScopeScreen_SetPixel(x0 + x, y0 + y);
    UB_ScopeScreen_SetPixel(x0 - x, y0 + y);
    UB_ScopeScreen_SetPixel(x0 + x, y0 - y);
    UB_ScopeScreen_SetPixel(x0 - x, y0 - y);
    UB_ScopeScreen_SetPixel(x0 + y, y0 + x);
    UB_ScopeScreen_SetPixel(x0 - y, y0 + x);
    UB_ScopeScreen_SetPixel(x0 + y, y0 - x);
    UB_ScopeScreen_SetPixel(x0 - y, y0 - x);
  }
}


//--------------------------------------------------------------
// Zeichnet ein Ascii-Zeichen eines Fonts an x,y Position
// (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
void UB_ScopeScreen_DrawChar(uint16_t x, uint16_t y, uint8_t ascii, UB_Font *font)
{
  uint16_t xn,yn,start_maske,maske;
  const uint16_t *wert;


  ascii -= 32;
  wert=&font->table[ascii * font->height];

  start_maske=0x80;
  if(font->width>8) start_maske=0x8000;

  for(yn = 0; yn < font->height; yn++) {
    maske=start_maske;
    for(xn = 0; xn < font->width; xn++) {
      if((wert[yn] & maske) == 0x00) {
        // Pixel loeschen
        UB_ScopeScreen_ClearPixel(x+xn,y+font->height-yn);
      }
      else {
        // Pixel setzen
        UB_ScopeScreen_SetPixel(x+xn,y+font->height-yn);
      }
      maske=(maske>>1);
    }
  }
}


//--------------------------------------------------------------
// Zeichnet einen String eines Fonts an x,y Position
// (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
void UB_ScopeScreen_DrawString(uint16_t x, uint16_t y,char *ptr, UB_Font *font)
{
  uint16_t pos;

  pos=x;
  while (*ptr != 0) {
    UB_ScopeScreen_DrawChar(pos,y,*ptr,font);
    pos+=font->width;
    ptr++;
  }
}


//--------------------------------------------------------------
// berechnet aus dem Inhalt vom Scope-Screen die Pixel-Liste
// Hinweis : diese Funktion muss jedesmal aufgerufen werden,
//           wenn sich der Bildinhalt gendert hat)
//
// ret_wert : SUCCESS -> ok
//            ERROR   -> zu viele Pixel sind gesetzt (Array OVF)
//
// die Anzahl der Pixel die gesetzt sind, steht in
// "ScopeScreen.pixel_anz"
//--------------------------------------------------------------
ErrorStatus UB_ScopeScreen_RedrawScreen(void)
{
  ErrorStatus ret_wert=SUCCESS;
  uint16_t x,y;
  uint8_t mode=0;

  // warte auf Sync-Signal
  while(ScopeScreen.akt_list_pos!=0);

  // komplettes Bild nach gesetzten Pixeln durchsuchen
  ScopeScreen.pixel_anz=0;
  for(y=0;y<SCOPE_SCREEN_MAXY;y++) {
    if(mode==0) {
      // von links nach rechts
      mode=1;
      for(x=0;x<SCOPE_SCREEN_MAXX;x++) {
        if(p_ScopeScreen_check_pixel(x,y)!=0) {
          if(ScopeScreen.pixel_anz<SCOPE_MAX_PIXEL) {
            SCOPE_PIXEL_LIST[ScopeScreen.pixel_anz].xpos=x;
            SCOPE_PIXEL_LIST[ScopeScreen.pixel_anz].ypos=y;
            ScopeScreen.pixel_anz++;
          }
          else {
            ret_wert=ERROR;
          }
        }
      }
    }
    else {
      // von rechts nach links
      mode=0;
      for(x=SCOPE_SCREEN_MAXX;x>0;x--) {
        if(p_ScopeScreen_check_pixel(x-1,y)!=0) {
          if(ScopeScreen.pixel_anz<SCOPE_MAX_PIXEL) {
            SCOPE_PIXEL_LIST[ScopeScreen.pixel_anz].xpos=x-1;
            SCOPE_PIXEL_LIST[ScopeScreen.pixel_anz].ypos=y;
            ScopeScreen.pixel_anz++;
          }
          else {
            ret_wert=ERROR;
          }
        }
      }
    }
  }

  if(ScopeScreen.pixel_anz==0) {
    // falls keine Pixel gesetzt sind
    SCOPE_PIXEL_LIST[0].xpos=0;
    SCOPE_PIXEL_LIST[0].ypos=0;
  }

  // Pixel anzahl setzen
  ScopeScreen.akt_pixel_anz=ScopeScreen.pixel_anz;

  return(ret_wert);
}


//--------------------------------------------------------------
// zeichnet ein Testbild auf den Scope-Screen
// (bestehend aus Linien und Kreisen mit ca. 1500 Pixel)
//--------------------------------------------------------------
void UB_ScopeScreen_Testpattern(void)
{
  // loescht den Screen
  UB_ScopeScreen_Clear();

  // zeichnet eine umrandung aus 4 linien
  UB_ScopeScreen_DrawLine(0,0,SCOPE_SCREEN_MAXX-1,0);// linie unten
  UB_ScopeScreen_DrawLine(0,SCOPE_SCREEN_MAXY-1,SCOPE_SCREEN_MAXX-1,SCOPE_SCREEN_MAXY-1);// linie oben
  UB_ScopeScreen_DrawLine(0,0,0,SCOPE_SCREEN_MAXY-1);// linie links
  UB_ScopeScreen_DrawLine(SCOPE_SCREEN_MAXX-1,0,SCOPE_SCREEN_MAXX-1,SCOPE_SCREEN_MAXY-1);// linie rechts

  // zeichnet zwei schrgen
  UB_ScopeScreen_DrawLine(0,0,SCOPE_SCREEN_MAXX-1,SCOPE_SCREEN_MAXY-1);
  UB_ScopeScreen_DrawLine(0,SCOPE_SCREEN_MAXX-1,SCOPE_SCREEN_MAXX-1,0);

  // zeichnet einen Kreis in der Mitte
  UB_ScopeScreen_DrawCircle(SCOPE_SCREEN_MAXX/2,SCOPE_SCREEN_MAXY/2,SCOPE_SCREEN_MAXX/4);

  // Screen neu zeichnen
  UB_ScopeScreen_RedrawScreen();
}


//--------------------------------------------------------------
// interne Funktion
// test ob Pixel an x,y gesetzt oder nicht
// xpos : [0...SCOPE_SCREEN_MAXX-1]
// ypos : [0...SCOPE_SCREEN_MAXY-1]
//
// ret_wert : 0 = pixel nicht gesetzt
//            1 = pixel gesetzt
//--------------------------------------------------------------
uint8_t p_ScopeScreen_check_pixel(uint16_t xpos, uint16_t ypos)
{
  uint8_t ret_wert=0;
  uint32_t adr,pos;
  uint8_t maske=0;

  if(xpos>=SCOPE_SCREEN_MAXX) xpos=SCOPE_SCREEN_MAXX-1;
  if(ypos>=SCOPE_SCREEN_MAXY) ypos=SCOPE_SCREEN_MAXY-1;

  adr=(ypos*SCOPE_SCREEN_RAM_XLEN)+(xpos>>3);

  pos=(xpos&0x07);
  if(pos==0) maske=0x01;
  if(pos==1) maske=0x02;
  if(pos==2) maske=0x04;
  if(pos==3) maske=0x08;
  if(pos==4) maske=0x10;
  if(pos==5) maske=0x20;
  if(pos==6) maske=0x40;
  if(pos==7) maske=0x80;

  if((SCOPE_SCREEN_RAM[adr]&maske)==0) {
    ret_wert=0;
  }
  else {
    ret_wert=1;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Signum funktion
//  Return_Wert
//    1 ,wenn x > 0
//    0 ,wenn x = 0
//   -1 ,wenn x < 0
//--------------------------------------------------------------
int16_t p_ScopeScreen_sgn(int16_t x)
{
  return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}


//--------------------------------------------------------------
// interne Funktion
// init vom Timer
//--------------------------------------------------------------
void p_ScopeScreen_init_TIM(void)
{
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

  // Clock enable
  RCC_APB1PeriphClockCmd(SCOPE_TIMER_CLK, ENABLE);

  // Timer disable
  TIM_Cmd(SCOPE_TIMER, DISABLE);

  // Timer init
  TIM_TimeBaseStructure.TIM_Period =  SCOPE_TIM_PERIODE;
  TIM_TimeBaseStructure.TIM_Prescaler = SCOPE_TIM_PRESCALE;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(SCOPE_TIMER, &TIM_TimeBaseStructure);

  // Timer preload enable
  TIM_ARRPreloadConfig(SCOPE_TIMER, ENABLE);
}


//--------------------------------------------------------------
// interne Funktion
// init vom NVIC
//--------------------------------------------------------------
void p_ScopeScreen_init_NVIC(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  // Update Interrupt enable
  TIM_ITConfig(SCOPE_TIMER,TIM_IT_Update,ENABLE);

  // NVIC konfig
  NVIC_InitStructure.NVIC_IRQChannel = SCOPE_TIMER_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}


//--------------------------------------------------------------
// interne Funktion
// Anzeigen eines Pixels vom Scope-Screen (per DAC1 und DAC2)
// diese Funktion wird zyklisch per Timer-Interrupt aufgerufen
//--------------------------------------------------------------
void p_ScopeScreen_DrawNextPixel(void)
{
  static uint8_t mode=0;
  uint16_t x,y;

  // alternierend anzeigen
  if(mode==0) {
    // von unten nach oben
    x=SCOPE_PIXEL_LIST[ScopeScreen.akt_list_pos].xpos*SCOPE_FAKTOR_X;
    y=SCOPE_PIXEL_LIST[ScopeScreen.akt_list_pos].ypos*SCOPE_FAKTOR_Y;
    UB_DAC_SetDAC1u2(x,y);

    ScopeScreen.akt_list_pos++;
    if(ScopeScreen.akt_list_pos>=ScopeScreen.akt_pixel_anz) {
      mode=1;
    }
  }
  else {
    // von oben nach unten
    x=SCOPE_PIXEL_LIST[ScopeScreen.akt_list_pos-1].xpos*SCOPE_FAKTOR_X;
    y=SCOPE_PIXEL_LIST[ScopeScreen.akt_list_pos-1].ypos*SCOPE_FAKTOR_Y;
    UB_DAC_SetDAC1u2(x,y);

    ScopeScreen.akt_list_pos--;
    if(ScopeScreen.akt_list_pos==0) {
      mode=0;
    }
  }
}


//--------------------------------------------------------------
// Timer-Interrupt
// wird beim erreichen vom Counterwert aufgerufen
//--------------------------------------------------------------
void SCOPE_TIMER_ISR(void)
{
  if (TIM_GetITStatus(SCOPE_TIMER, TIM_IT_Update) != RESET) {
    // wenn Interrupt aufgetreten
    TIM_ClearITPendingBit(SCOPE_TIMER, TIM_IT_Update);

    // ein Pixel vom Scope-Screen zeichnen
    p_ScopeScreen_DrawNextPixel();
  }
}

